#!/bin/sh
#------------------------------------------------------------------------------
# SSL fix test installer - by fantogni - v 1.02
# Checks if device is compatible with upgrade/update
#
# Usage:
#   ./test-install.sh
#
# Returns: an error code
#       0: OK, installation can be done (returned by help too)
#       1: generic error
#       2: (reserved by bash)
#       3: SSL fix already installed
#       4: System not recognized
#       5: Test of new libraries failed
#
# Notes:
# Checks whether the updated files will work on the target system. Will not
# make any attempt at modifying the firmware.
# To be used with non Chumby One devices.
#
# Version 1.02:
#   Fixed bug which prevented test from working in some cases
#------------------------------------------------------------------------------
# Copyright Notice
# This software is free to modify and reuse.
# If you find it useful, please give credits in your code to the original author.
#
# -----------------------------------------------------------------------------

VERSION="1.02"

LIB="/lib"
BIN="/usr/bin"
CERT="/usr/share/certs"
SRC="/mnt/usb/src"

INST_R='F'
INST_P='P'          #Partially recoverable
INST_O='F'

LIBCURL_VO="libcurl.so.4.0.0"
LIBCURL_V1="${LIBCURL_VO}"
LIBCURL_SL="libcurl.so"
LIBCURL_SL4="libcurl.so.4"
BINCURL="curl"
CERTS="curl-ca-bundle.crt"


OPENSSL_VO="OpenSSL/0.9.7p"
OPENSSL_V1="OpenSSL/1.0.0s"
OPENSSL_V2="OpenSSL/1.0.2l"
SSL_V3="mbedTLS/2.1.9"

LIBCURL_BKP="${LIBCURL_VO}_bkp"
BINCURL_BKP="${BINCURL}_bkp"
CERTS_BKP="${CERTS}_bkp"

case $1 in
  '-?'|--help)
    echo "v $VERSION"
    echo "Usage: $0"
	echo "    : checks compatibility of device"
    echo "  -?|--help: this help screen"
    exit 0 ;;

  '')
    ;;

 *)
    echo "Unrecognized option $1"
    exit 0
esac                             


# Functions

checkSymLink() {                 
  ( ls -l $1 | grep -q $2 )
}

echoSymLinkRef() {
  ex="s#^.+$1 [^ ]+ (.+)\$#\1# p"
  ls -l "$1"|sed -nr "$ex"
}

checkCurlSSL() {
  ossl=$( curl -V | sed -nr 's/^.+(mbedTLS\/[0-9\.]+).+$/\1/ p;')
  if [ "$ossl" == "$1" ] ; then
    return 0;
  fi
  return 1
}

# checkFSWritable() {
  # rt=$( mount |sed -nr  's#^/dev/root.+\(([a-z]+)\,.+$#\1# p' )
  # if [ "$rt" = "rw" ] ; then exit 0 ; fi
  # exit 1
# }

          

testUpdate() {
  if ( ${SRC}/curl "https://www.wsj.com/xmll/rss/3_7084.xml" 1>/dev/null 2>&1 ) &&\
     ( ${SRC}/curl -o "/tmp/mountain.jpg" "https://img.engadin.stmoritz.ch/object/6308152/400x.jpg" 2>/dev/null ) 
  then
    echo "Test of update successfull"
    return 0
  fi
  echo "Test failed: unable to load secured content"
  return 1
}
		  
echo "Checking installation"

# Check if it's recoverable
if [ -e "${BIN}/${BINCURL_BKP}" ] && \
   [ -e "${LIB}/${LIBCURL_BKP}" ] && \
   [ -e "${CERT}/${CERTS_BKP}" ] 
then
  INST_R='T'
  INST_P='T'
  echo "Recoverable installation"
else
  if [ -e "${LIB}/$LIBCURL_BKP" ] ; then
    echo "Partially recoverable installation"
    INST_P="T"
  else
    echo "Not recoverable installation"
  fi
fi

#Check if it's the original setup 
if ( checkSymLink "${LIB}/${LIBCURL_SL}"  "${LIBCURL_VO}" ) && \
   ( checkSymLink "${LIB}/${LIBCURL_SL4}" "${LIBCURL_VO}" ) && \
   [ -e "${BIN}/${BINCURL}" ] && \
   [ -e "${LIB}/${LIBCURL_VO}" ] && \
   ! [ -e "${BIN}/${BINCURL_BKP}" ] && \
   ! [ -e "${LIB}/${LIBCURL_BKP}" ]
   # ! [ -e "${LIB}/${LIBCURL_V2}" ] &&\		BUG if LIBCURL_Vn is undefined this will always fail!!
   # ! [ -e "${LIB}/${LIBCURL_V3}" ]
then
  echo "Original installation"
  INST_O='T'
fi

#DEBUG remove next line
#INST_O='T'

if [ "${INST_O}" = "T" ] ; then
	LD_PATH=${LD_LIBRARY_PATH}
	ex="0"
	export LD_LIBRARY_PATH="${SRC}:${LD_LIBRARY_PATH}"
	${SRC}/curl -V
	if ( testUpdate ) ; then
		echo "Update can be successfully installed"
	else
		echo "Unfortunately update will not work and should not be installed"
		ex="5"
	fi
	export LD_LIBRARY_PATH="${LD_PATH}"
	echo $LD_LIBRARY_PATH
	exit $ex
elif [ "${INST_R}" = "T" ] ; then
	echo "SSL fix already installed."
	exit 3
fi
echo "Update will not be installed because the system wasn't recognized"
exit 4


######################################## TODO
## Remove stuff below
##

#Check whether it's Version 1
if ( checkSymLink "${LIB}/${LIBCURL_SL}" "$LIBCURL_V1" ) && \
   ( checkSymLink "${LIB}/${LIBCURL_SL4}" "$LIBCURL_V1" ) && \
   [ -e "${BIN}/${BINCURL}" ] && \
   [ -e "${BIN}/${BINCURL_BKP}" ] && \
   [ -e "${LIB}/$LIBCURL_V1" ] && \
   [ -e "${LIB}/$LIBCURL_BKP" ] && \
   [ -e "${CERT}/${CERTS}" ]&& \
   [ -e "${CERT}/${CERTS_BKP}" ] 
then
  echo "V1 installed"
  INST_1='T'
else                             
  echo "V1 not installed"                   
fi

#Check whether it's Version 2
if ( checkSymLink "${LIB}/${LIBCURL_SL}" "$LIBCURL_V2" ) && \
   ( checkSymLink "${LIB}/${LIBCURL_SL4}" "$LIBCURL_V2" ) && \
   [ -e "${BIN}/${BINCURL}" ] && \
   [ -e "${LIB}/$LIBCURL_V2" ] && \
   [  "${INST_R}" = "T" ]
then
  echo "V2 installed"
  INST_2='T'
else                             
  echo "V2 not installed"                   
fi

#Check whether it's Version 3
if ( checkSymLink "${LIB}/${LIBCURL_SL}" "$LIBCURL_V3" ) && \
   ( checkSymLink "${LIB}/${LIBCURL_SL4}" "$LIBCURL_V3" ) && \
   [ -e "${BIN}/${BINCURL}" ] && \
   [ -e "${LIB}/$LIBCURL_V3" ] && \
   [  "${INST_R}" = "T" ]
then
  echo "V3 installed"
  INST_3='T'
else                             
  echo "V3 not installed"                   
fi

#This is confusing: was useful during beta testing now I don't know
#if [ "$INST_1$INST_2" == "TT" ] ; then
#  echo "mixed installation, recoverable"
#fi

if checkCurlSSL $SSL_V3 ; then
  echo "mbedtls (already) updated to V3"
fi

if [ "$INST" = "I" ] ; then
  res=""
  if [ "$INST_O" = "T" ] ; then
    echo "Installing V3"
    if ( installV3 ) ; then
	  res="S"
	else
	  res="F"
	fi
  elif 	[ "${INST_1}${INST_2}${INST_3}" = "TFF" ] ||\
		[ "${INST_1}${INST_2}${INST_3}" = "FTF" ]  ; then
    echo "Upgrading V1/V2 to V3"
    if ( upgradeV3fromV ) ; then
	  res="S"
	else
	  res="F"
    fi
	## Here goes any V3 Update 
	## (upgrade: changes libcurl version | update: keeps libcurl version)
  fi 
  if [ "$res" = "S" ] ; then 
    echo "Installation of V3 successfull"
    if ( testV3 ) ; then
      curl -V
	  exit 0
	else
	  echo "Test of V3 failed, running restore is strongly recommended.".
	  exit 1
	fi
  elif [ "$res" = "F" ] ; then
    echo "Error while installing, installation incomplete. Run restore"
	exit 1
  fi
fi

if [ "$INST_P" = "T" -a "$INST" = "U" ] ; then
  echo "Recovering original installation"
  if recoverVx ; then
    echo "Uninstall successfull"
	exit 0
  else
    echo "Error while recovering, original setup might not have been restored."
  fi
fi
if ! [ -z $INST ] ; then 
  echo "Unable to perform install/recover command"
fi
